## Notice d'utilisation

### Tester

Télécharger les fichiers d'exemple et les ouvrir dans l'application :

- [Paramètres](tests/param-test1.csv)
- [Voeux](tests/voeux-test1.csv)

L'édition de ces fichiers se fait de préférence avec un tableur, mais est 
également possible avec un éditeur de texte.

### Utilisation

- Charger le fichier des vœux élèves. Le logiciel calcule automatiquement 
le nombre de groupe à créer dans chaque spécialité, crée les alignements et 
répartit les élèves.
- Charger le fichier des paramètres pour ajuster la taille et le nombre de 
groupes (par exemple pour définir des groupes réduits, en langues ou sciences
par exemple).

Analyser le résultat (existence ou non de vœux non satisfaits, coût en heures,
équilibrage des groupes), puis expérimenter éventellement de nouveaux paramètres 
(il suffit de redéposer / resélectionner un fichier pour mettre à jour la 
solution) :

- fermer une spécialité ou limiter le nombre de groupes pour une spécialité 
(certains élèves n'auront pas leur vœux satisfaits);
- augmenter ou réduire l'effectif maximum d'un groupe :

	- pour réduire ou augmenter le nombre de groupes ;
	- en cas de déséquilibre entre les effectifs des différents groupes d'une 
	même spécialité (par exemple, pour une moyenne de 28 élèves par groupe, 
	essayer une taille entre 29 et 31). 

### Export

Télécharger si besoin la solution au format CSV; cette fonctionnalité permet :

- de réordonner spécialités ou élèves pour optimiser manuellement la solution ;
- de faciliter l'interopérabilité avec d'autres logiciels.



## Fichier de vœux des élèves

Le logiciel charge les données à partir d'un fichier <abbr title="Comma-Separated Values">CSV</abbr> 
dont les lignes respectent le format suivant :

| Nom      | Prénom  | Spé.1 | Spé.2 | Spé.3   | Spé.3' |
| -------- | ------- | ----- | ----- | ------- | ------ |
| Barjavel | René    | PC    | HGGSP | Maths   | SVT    |
| Bordage  | Pierre  | LLCE  | HLP   | Théatre | HGGSP  |
| Verne    | Jules   | Maths | NSI   | PC      | SVT    |
| Werbert  | Bernard | SES   | SVT   | HLP     | LLCE   |

- Pour les vœux de terminale, il faut supprimer les colonnes Spécialité 3 
et Spécialité 3' (sinon le logiciel pense que c'est un fichier pour des premières
et crée 3 alignements).
- Spécialité 3' est un second choix (facultatif) dans le cas ou le troisième 
vœu (de première) ne pourrait être satisfait. En pratique, c'est surtout utile 
pour les élèves choisissant une spécialité dont on n'est pas sûr de l'ouverture. 
Attention, dans ce cas, les vœux des spécialités 1 et 2 sont considérés comme 
fermes ; seule la spécialité 3 peut être remplacée par la 3'.

Un fichier CSV peut facilement être importé, modifié et exporté à partir
d'un tableur comme Excel ou LibreOffice-Calc.



## Fichier de paramétrage

Le fichier de paramétrage a un impact important sur la répartition des élèves.

### Rôle principal du fichier de paramétrage

Le fichier de paramétrage permet de spécifier :

- une taille de groupe classe différente de 35,
- l'existence de dédoublements (effectifs réduits),
- une limite en nombre de groupes (ou la fermeture d'une spécialité).

Le format de ce fichier CSV est le suivant :

| Spécialité      | Effectifs classe | Effectifs réduits | Limite |
| --------------- | ----------------:| -----------------:| ------:|
| SVT             | 35               | 24                | -1     |
| LLCE-Anglais    | 24               | 0                 | -1     |
| Arts-plastiques | 24               | 0                 | 1      |
| SI              | 35               | 0                 | 0      |

#### Précisions :

- Le nom de la spécialité doit avoir la même orthographe que dans le fichier 
des vœux.
- Effectifs classe et effectifs réduit sont respectivement les effectifs maximums 
en classe entière et non dédoublée (capacité des salles de travaux pratiques 
par exemple) ; si effectifs reduits est égal à 0, le logiciel considère que
pour cette spécialité, les élèves sont toujours en classe entière.
- La limite permet de limiter le nombre de groupes (et donc le nombre d'élèves 
pouvant suivre cette spécialité) ; une valeur de "-1" signifie qu'autant de 
groupes que nécessaire seront créés.

#### L'exemple permet :

- de définir pour la spécialité SVT des groupes de 24 élèves pour les travaux 
pratiques ;
- de limiter la taille des groupes à 24 pour les spécialités arts plastiques 
et LLCE ;
- de limiter à 1 le nombre de groupes d'arts plastiques ;
- de fermer la spécialité SI (Sciences de l'Ingénieur).

#### Remarque :

Lorsqu'un cours est dédoublé (pour des séances de travaux pratiques par exemple), 
il peut être intéressant (surcoût maximum de 2h seulement) de n'avoir que 
des classes non dédoublées de 24 élèves plutôt qu'une classe de 35. En effet, 
avec dédoublement, il y a 2 heures (de travaux pratiques, ...) à placer à 
un autre moment de l'emploi du temps ou à faire assurer par un autre professeur 
(pour le second groupe de TP). C'est pour cette raison que l'algorithme privilégie 
les classes non dédoublées (à coût horaire équivalent).

#### Conseils :

- Ne pas utiliser la colonne effectif réduit (laisser la valeur à 0) ; ce 
paramétrage pourrait être supprimé dans une version ultérieure en raison de 
son intérêt limité au vu des contraintes que cela implique sur l'emploi du 
temps.
- Pour l'effectif, il peut être intéressant d'estimer l'effectif moyen par 
groupe en fonction du nombre d'élèves ayant demandé cette spécialité. Par exemple, 
si 57 élèves demandent une spécialité pour laquelle l'effectif maximum est 
de 24 élèves, 3 groupes sont nécéssaires (soit 72 places disponibles). Sans 
contrainte, l'algorithme n'arrive pas toujours à équilibrer les effectifs des 
groupes (par exemple, un groupe de 24 élèves, un de 12 et le dernier de 21). 
On peut essayer de le forcer à équilibrer les effectifs en limitant la taille 
du groupe à l'effectif moyen (ici 19), ou plus raisonnablement en ajoutant 
1 ou 2 à cette valeur.


### Rôle secondaire du fichier de paramétrage

La compréhension du mécanisme de construction des alignements permet d'envisager 
des solutions lorsque l'algorithme n'y parvient pas.

Dans la zone "Structure des spécialités", ces dernières sont ordonnées : Le 
premier groupe de la première spécialité est placé dans le premier alignement,
le second groupe de cette même spécialité dans le second ... Lorsque tous 
les groupes d'une spécialité ont été traités, on passe à la suivante. Le premier 
groupe de cette spécialité est placé dans le prochain alignement (par exemple 
le troisième si le dernier groupe de la spécialité précédente a été placé 
dans le deuxième). Exemple, pour la structure suivante :

- NSI, 2 groupes
- Maths, 3 groupes
- SES, 2 groupes

les alignements (en première) seront :

- Alignement 1 :

	- NSI-1 (placé en premier)
	- Maths-2 (placé en quatrième)
	- SES-2

- Alignement 2 :

	- NSI-2 (placé en deuxième)
	- Maths-3 (placé en cinquième)
	
- Alignement 3 :

	- Maths-1 (placé en troisième)
	- SES-1

L'algorithme respecte l'ordre des spécialités indiqué dans le fichier des 
paramètres, à moins de trouver une meilleure solution grâce à un calcul de
pondération qui dépend :

- du nombre de groupe : moins il y en a, plus le poids est élevé,
- et du nombre de places restantes dans les spécialités : si 57 élèves demandent 
une spécilité pour laquelle 3 groupes de 19 sont crées, le poids est supérieur 
à la situation la taille des groupes est de 24 (pour le même nombre d'élèves).

Ainsi, la recherche d'une solution satisfaisante passe par l'expérimentation  
de plusieurs solutions ; l'expérience montre qu'il est a priori plus simple 
de jouersur les effectifs de classe, mais qu'il est parfois nécessaire de 
réordonner les spécialités dans le fichier des paramètres.

*Note : l'amélioration de l'ergonomie pour permettre ces ajustements est une
des choses à faire pour les versions futures.*

